@@ -1,7 +1,6 @@
 
 /*
  * Copyright (C) Igor Sysoev
- * Copyright (C) Nginx, Inc.
  */
 
 
@@ -112,8 +111,8 @@ static ngx_http_module_t  ngx_http_limit_req_module_ctx = {
     NULL,                                  /* create server configuration */
     NULL,                                  /* merge server configuration */
 
-    ngx_http_limit_req_create_conf,        /* create location configuration */
-    ngx_http_limit_req_merge_conf          /* merge location configuration */
+    ngx_http_limit_req_create_conf,        /* create location configration */
+    ngx_http_limit_req_merge_conf          /* merge location configration */
 };
 
 
@@ -372,42 +371,47 @@ ngx_http_limit_req_lookup(ngx_http_limit_req_conf_t *lrcf, ngx_uint_t hash,
 
         /* hash == node->key */
 
-        lr = (ngx_http_limit_req_node_t *) &node->color;
+        do {
+            lr = (ngx_http_limit_req_node_t *) &node->color;
 
-        rc = ngx_memn2cmp(data, lr->data, len, (size_t) lr->len);
+            rc = ngx_memn2cmp(data, lr->data, len, (size_t) lr->len);
 
-        if (rc == 0) {
-            ngx_queue_remove(&lr->queue);
-            ngx_queue_insert_head(&ctx->sh->queue, &lr->queue);
+            if (rc == 0) {
+                ngx_queue_remove(&lr->queue);
+                ngx_queue_insert_head(&ctx->sh->queue, &lr->queue);
 
-            tp = ngx_timeofday();
+                tp = ngx_timeofday();
 
-            now = (ngx_msec_t) (tp->sec * 1000 + tp->msec);
-            ms = (ngx_msec_int_t) (now - lr->last);
+                now = (ngx_msec_t) (tp->sec * 1000 + tp->msec);
+                ms = (ngx_msec_int_t) (now - lr->last);
 
-            excess = lr->excess - ctx->rate * ngx_abs(ms) / 1000 + 1000;
+                excess = lr->excess - ctx->rate * ngx_abs(ms) / 1000 + 1000;
 
-            if (excess < 0) {
-                excess = 0;
+                if (excess < 0) {
+                    excess = 0;
+                }
+
+                *ep = excess;
+
+                if ((ngx_uint_t) excess > lrcf->burst) {
+                    return NGX_BUSY;
+                }
+
+                lr->excess = excess;
+                lr->last = now;
+
+                if (excess) {
+                    return NGX_AGAIN;
+                }
+
+                return NGX_OK;
             }
 
-            *ep = excess;
+            node = (rc < 0) ? node->left : node->right;
 
-            if ((ngx_uint_t) excess > lrcf->burst) {
-                return NGX_BUSY;
-            }
+        } while (node != sentinel && hash == node->key);
 
-            lr->excess = excess;
-            lr->last = now;
-
-            if (excess) {
-                return NGX_AGAIN;
-            }
-
-            return NGX_OK;
-        }
-
-        node = (rc < 0) ? node->left : node->right;
+        break;
     }
 
     *ep = 0;
@@ -566,8 +570,6 @@ ngx_http_limit_req_merge_conf(ngx_conf_t *cf, void *parent, void *child)
 
     if (conf->shm_zone == NULL) {
         conf->shm_zone = prev->shm_zone;
-        conf->burst = prev->burst;
-        conf->nodelay = prev->nodelay;
     }
 
     ngx_conf_merge_uint_value(conf->limit_log_level, prev->limit_log_level,
